from shangqi_cloud_lib.utils.ElasticsearchUtil import es_condition_by_match_phrase, es_condition_by_terms


def common_format_list_tags(result_tags, insert_tags, tags_index=10):
    if insert_tags is not None and len(insert_tags) > 0:
        insert_tag_list = insert_tags[0:tags_index]
        for tag in insert_tag_list:
            if isinstance(tag, str):
                if tag and tag.strip() != "":
                    result_tags.append(tag)


def common_format_string_tags(result_tags, data, key):
    if data.get(key) is not None and data.get(key) != "":
        result_tags.append(data.get(key))
    return result_tags


"""

# 政策通 政策标签
def format_policy_tags(policy):
    shangqi_tags = policy.get("tags",{})

    result_tags = {
        "support_object":policy.get("support_object", []),
        "support_way":policy.get("support_way", [])
    }

    common_format_list_tags(tags, )

    common_format_list_tags(tags, policy.get("support_behavior", []))
    common_format_string_tags(tags, policy, "is_declare")
    if shangqi_tags:
        industry_tags = [data["tag_name"] for data in shangqi_tags.get("industry_tag",[])]
        certification_tags = shangqi_tags.get("certification_tag",[])
        common_format_list_tags(tags, industry_tags)
        common_format_list_tags(tags, certification_tags)

    tags_list = [{
        "type": "other",
        "tabs": tags
    }]
    if policy.get("tags"):
        tags = policy["tags"]
        tags_list.append({
            "type": "产业类型",
            "tabs": [data["tag_name"] for data in tags.get("industry_tag", [])]
        })
    return tags_list


"""


def format_param_tag(bool_should_more_list, especial_tag_list):
    bool_should_list = []
    if especial_tag_list is not None:
        for tag in especial_tag_list:
            if tag in ["单项冠军", "小巨人", "瞪羚企业", "一条龙", "专精特新"]:
                es_condition_by_match_phrase(bool_should_list, "tags.national_tag.tag_name", tag)
            if tag in ["单项冠军", "隐形冠军", "成长", "小巨人", "瞪羚企业", "首台套", "专精特新", "雏鹰"]:
                es_condition_by_match_phrase(bool_should_list, "tags.province_tag.tag_name", tag)
            if tag in ["雏鹰"]:
                es_condition_by_match_phrase(bool_should_list, "tags.city_tag.tag_name", tag)
            if tag in ["雏鹰"]:
                es_condition_by_match_phrase(bool_should_list, "tags.district_tag.tag_name", tag)
            if tag in ["独角兽", "中国企业500强"]:
                es_condition_by_match_phrase(bool_should_list, "tags.rank_tag.rank_name", tag)
            if tag in ["瞪羚企业"]:
                es_condition_by_match_phrase(bool_should_list, "tags.park_tag.tag_name", tag)
            if tag == "高新技术企业":
                es_condition_by_match_phrase(bool_should_list, "tags.certification.certification_name.keyword", tag)
            if tag == "科技型中小企业":
                es_condition_by_match_phrase(bool_should_list, "tags.certification.certification_name.keyword", tag)
            if tag in ["已上市", "排队上市", "已退市"]:
                es_condition_by_terms(bool_should_list, "tags.market_tag.status.keyword", [tag])
            if tag in ["主板上市", "创业板上市", "科创板上市", "新三板-基础层", "新三板-创新层", "新三板-精选层", "北交所"]:
                es_condition_by_terms(bool_should_list, "tags.market_tag.block.keyword", [tag])
            if tag in ["规上企业"]:
                es_condition_by_terms(bool_should_list, "tags.nonpublic_tag.keyword", [tag])

    bool_should_more_list.append(bool_should_list)


def format_company_shangqi_level(company_info):
    ent_rate_data = float(company_info.get("tags", {}).get("score_tag", {}).get("total_point", 0.0))
    if ent_rate_data is not None:
        if ent_rate_data < 40.0:
            rate = "A"
        elif ent_rate_data < 60.0:
            rate = "AA"
        elif ent_rate_data < 80.0:
            rate = "AAA"
        else:
            rate = "S"
    else:
        rate = "A"
    return rate



import types
from abc import ABC
from typing import Any

import pymysql
import tornado.web
import tornado.ioloop

import json

from tornado import httputil
from tornado.web import Application

from shangqi_cloud_lib.context import config
from shangqi_cloud_lib.frame.HandlerHelper import check_auth, command_mapper
from shangqi_cloud_lib.utils.CacheUtil import redis_connect
from shangqi_cloud_lib.utils.CommonUtil import CJsonEncoder, get_host_ip
from shangqi_cloud_lib.utils.DataClient import DataClient
from shangqi_cloud_lib.utils.FileUtil import file_upload
from shangqi_cloud_lib.utils.JwtUtil import auth, decode_token
from shangqi_cloud_lib.utils.MysqlUtil import query_update

ERRNO_OK = 0
ERRMSG_OK = "ok"
ERRNO_INVALID_PARAM = 801


class BaseHandler(tornado.web.RequestHandler, ABC):
    def __init__(self, application: "Application", request: httputil.HTTPServerRequest, **kwargs: Any):
        super().__init__(application, request, **kwargs)
        pymysql.install_as_MySQLdb()
        self.command_map = {}
        methods = self.methods()
        self.client = DataClient(f"http://{config.data_server_ip}:{config.data_server_port}", config.data_server_token)
        for i in range(len(methods)):
            method = getattr(self, methods[i])
            if getattr(method, "__req__", None):
                self.command_map[method.__command__] = method

    def methods(self):
        return (list(filter(
            lambda m: not m.startswith("__") and not m.startswith("_") and callable(getattr(self, m)) and
                      type(getattr(self, m, None)) == types.MethodType,
            self.__dir__())))

    @check_auth
    @command_mapper
    def get(self):
        pass

    @check_auth
    @command_mapper
    def post(self):
        pass

    def set_default_header(self):
        origin = self.request.headers.get("Origin")
        if origin:
            self.set_header("Access-Control-Allow-Origin", origin)
        else:
            self.set_header("Access-Control-Allow-Origin", "*")

        self.set_header("Access-Control-Allow-Headers", "x-requested-with,Authorization,Can-read-cache")
        self.set_header("Access-Control-Allow-Methods", "POST,GET,PUT,DELETE,OPTIONS")
        self.set_header("Access-Control-Expose-Headers", "Access-Token")
        self.set_header("Access-Control-Allow-Credentials", "true")
        if config.headers:
            for k, v in config.headers.items():
                self.set_header(k, v)
        return True

    def options(self):
        self.set_default_header()
        self.write({
            "errno": 0,
        })

    def write_custom_error(self, errno, errmsg):
        self.write({
            "errno": errno,
            "errmsg": errmsg,
        })

    def write_401(self):
        self.set_status(401)

    def write_json_data(self, data):
        self.write(json.dumps(data, cls=CJsonEncoder, ensure_ascii=False))

    def write_data(self, data):
        self.write(data)

    # 注册用户权限验证  判断token 和token中key 是否合法, cookie中是否有用户信息,默认有效期1月
    def set_current_user(self, username):
        self.set_secure_cookie(name=config.app_scene, value=username, expires_days=config.cookie_expires_days,
                               domain=config.cookie_domain)

    def get_current_user(self) -> Any:
        username = self.get_secure_cookie(config.app_scene)
        if isinstance(username, bytes) or isinstance(username, bytearray):
            username = username.decode("utf-8")
        return username

    def auth_login(self):
        white_ip_list = self.get_white_ip_list()
        if self.request.headers.get("X-Real-Ip", "") in white_ip_list:
            return True
        return auth(self.get_token(), self.current_user)

    def get_white_ip_list(self):
        white_ip_list = config.white_ip_list
        if white_ip_list is None:
            white_ip_list = []
        white_ip_list.extend([get_host_ip(), "127.0.0.1", "::1", "localhost"])
        return white_ip_list

    def get_user_name(self):
        user_info = self.get_user_info()
        return user_info.get("user_name", None)

    def get_user_info(self):
        token = self.get_token()
        token_info = decode_token(token)
        user_name = token_info.get("key", "")
        user_info = token_info.get("info", {})
        if user_info:
            user_info["user_name"] = user_name
        return user_info

    def get_token(self):
        token = self.request.headers.get("Authorization", None)
        if token is None:
            token = self.get_argument("token", "")
        return token

    def auth_sum_control(self, key, interface_sum=config.interface_sum,
                         check_interface_time_horizon=config.check_interface_time_horizon,
                         server_port=config.server_port):
        if key:
            if key in self.get_white_ip_list():
                return True
            r = redis_connect()
            redis_user_name_key = str(server_port) + "&" + key
            if r.get(redis_user_name_key) is None:
                r.set(redis_user_name_key, 1, ex=check_interface_time_horizon * 3600)
                return True
            else:
                if json.loads(r.get(redis_user_name_key).decode()) <= interface_sum:
                    r.incr(redis_user_name_key)
                    return True
        return False

    def can_read_cache(self):
        can_cache = self.request.headers.get("Can-read-cache", "True")
        if can_cache.lower() == "false":
            can_cache = False
        else:
            can_cache = True
        return can_cache

    def upload_file(self, file_key, file_checker=lambda x: True):
        file_metas = self.request.files.get(file_key, None)
        filename = file_metas[0]['filename']
        if not file_checker(filename):
            return self.write_data(self.result_err("文件上传失败：{} 文件格式不匹配！".format(filename)))
        return file_upload(file_metas, config.temp_path)

    @staticmethod
    def result_ok(data=None, errmsg=ERRMSG_OK):
        res = {
            "errno": ERRNO_OK,
            "errmsg": errmsg,
        }
        if data is not None:
            res["data"] = data
        return res

    @staticmethod
    def result_err(msg, errno=ERRNO_INVALID_PARAM):
        return {
            "errno": errno,
            "errmsg": msg,
        }

    @staticmethod
    def result_custom_ok(data, **kwargs):
        kwargs["errno"] = ERRNO_OK
        kwargs["errmsg"] = ERRMSG_OK
        kwargs["data"] = data
        return kwargs

    def insert_request_record(self, func_name, command, params, arguments):
        if not params:
            for key in arguments:
                arguments[key] = str(arguments[key][0], encoding="utf-8")
            params = arguments
        insert_request_param_list = [
            "('{}','{}','{}','{}')".format(func_name, self.request.path[1:], command,
                                           json.dumps(params, ensure_ascii=False))]

        request_param_sql = "INSERT INTO test_request_record (request_type,class_route,interface_name,params_str) VALUES {}".format(
            ",".join(insert_request_param_list))

        query_update(request_param_sql)
